#! /usr/bin/perl
#Name: R2-server.perl
#Project: Rover control
#Homepage: www.HomoFaciens.de
#Author Norbert Heinz
#Version: 1.0
#Creation date: 10.03.2012
#Copyright: (c) 2012 by Norbert Heinz
#This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program if not, see http://www.gnu.org/licenses/.
#
#Server script for R2-client.perl. Both scripts are used to operate a remote controlled rover and to do the picture upload from the webcam.

use strict;
use IO::Socket;

use FindBin qw($Bin);

#$|=1 ;

$SIG{'QUIT'} = 'interrupt';
$SIG{'INT' } = 'interrupt';
$SIG{'HUP' } = 'interrupt';
$SIG{'TRAP'} = 'interrupt';
$SIG{'ABRT'} = 'interrupt';
$SIG{'STOP'} = 'interrupt';

my $start_command_time=0;
my $last_action_time=0;
my $socket;
my $received_data;
my $send_data;
my $client_socket;
my $peer_address;
my $peer_port;
my $packet_count = 0;
my $server_port = "1000";
my $file_out = "bild01.tmp";
my $file_command = "R2command.txt";
my $file_access = "R2access.txt";
my $file_server_status = "R2-connect.txt";
my $command_line = "";
my $fertig = 0;
my $file_size = 0;
my $single_char;
my $picture_count = 0;
my $send_data;
my $aut_ok = 0;
my $shell_command = "";
my $shell_command_02 = "";
my $char_temp = "";
my @file_info;
my $job_status = 0;
my $current_user = "";
my $current_ip = "";
my $current_timestamp = "";
my $current_snapshot = 0;
my $total_user_count = 0;
my $total_drive_distance = 0;
my $drive_distance = 0;
my ($seconds, $minutes, $hours, $day_of_month, $month, $year, $weekday, $day_of_year, $summertime) = localtime(time);
my $CTIME_String = localtime(time);
my $wait_count = 0;
my $last_watchdog_call = 0;
my $last_snapshot_name = "";
my $passwd_line = "";
my $last_indication_pic_no = 1;

my $file_temp = "history.tmp";
my $file_target = "";
my $file_in01 = "access-history01.log";
my $file_in02 = "access-history.log";
my $line_in ="";
my $user = "";
my $user_time = "";
my $user_pic = "";
my $user_count = 0;
my $total_user = 0;
my $total_distance = 0;
my $i = 0;
my $i2 = 0;

my $battery_status = 0;
my $battery_status_old = 999;

sub interrupt {#catch interrupt signals
  my($signal)=@_;
  &LogFile("Caught Interrupt\: $signal");
  print "Exiting program\n";
  `cp no_connection.jpg bild01.jpg`;
  `cp module_000.txt IrModule.txt`;
  &CopyIndicationPicture();
  exit(1);
}

print "\n\n";

chdir("$Bin");

  `cp no_connection.jpg bild01.jpg`;
  `cp module_000.txt IrModule.txt`;

  if (-e "access-count.log"){
    open(FILEIN, "< access-count.log") || die "Can not open file access-count.log!\n";
      $total_user_count = <FILEIN>;
      $total_drive_distance = <FILEIN>;
    close (FILEIN);
  }
  else{
    &LogFile ("Could not find access-count.log.");
  }

  &LogFile ("Starting R2 server");

  $socket = new IO::Socket::INET (
#                                    LocalHost => '192.168.1.2',
                                    LocalHost => 'www.XYZ.XX',
                                    LocalPort => $server_port,
                                    Proto => 'tcp',
                                    Listen => 1,    #number of clients to accept
                                    Reuse => 1
                                );

  die "Could not open socket" unless $socket;


while (111 == 111){

  #Tells R2-status.pl that server is waiting for connection
  open(FILEOUT, ">" . $file_server_status) || die "Can not open file $file_server_status!\n";
    print FILEOUT "Waiting for connection.\n";
  close (FILEOUT);

  $job_status = 0;

  print "Waiting for client on port " . $server_port . "\n";
  &WatchDogFile(1);
  `cp wait_for_connection.jpg bild01.jpg`;
  &CopyIndicationPicture();

  $client_socket = "";
  $client_socket = $socket->accept();

  $peer_address = $client_socket->peerhost();
  $peer_port = $client_socket->peerport();

  &LogFile ("Incomming connection from address '" . $peer_address . "', port '" . $peer_port . "'");

  $aut_ok=1;

  &ReceiveData(33);

  $passwd_line = substr("Type your password here" . "                              ", 0, 32);

  if(substr($received_data, 0, 32) eq $passwd_line){
     $aut_ok = 1;
     print "Authentication OK\n"
  }
  else{
    $aut_ok = 0;
    &LogFile ("Authentication failed! '" . $received_data . "' from address '" . $peer_address . "', port'" . $peer_port . "'");
  }

  #remove old commands if file ist still there
  $shell_command = "rm " . $file_command;
  `$shell_command`;

  #Clear file user access
  open(FILEOUT, ">" . $file_access) || die "Can not open file " . $file_command . "!\n";
  close(FILEOUT);
  $start_command_time = time();

  `cp connection_established.jpg bild01.jpg`;
  &CopyIndicationPicture();

  #Tell R2-status.pl that connection has been established
  `rm $file_server_status`;

  while($aut_ok == 1){

    &WatchDogFile(0);

    if($job_status != 0){
      print "job_status != 0\n";

      &ReceiveData(20);
      $file_size=substr($received_data, 12, 13);

      print "received data: '" . $received_data . "'\n";

      if($file_size < 1){
        &LogFile ("...file_size 0, ending connection");
        $aut_ok = 0;
      }
      else{
        #Receive picture from R2
        open(FILEOUT,'>'.$file_out) || die "Can not open file " . $file_out . "!\n";

        $packet_count = 0;

        while ($file_size > 0 && $aut_ok == 1){
          $last_action_time = time();
          &WatchDogFile(0);
          $received_data = "";
          if($file_size < 1024){
            &ReceiveData(1024);
            $received_data = substr($received_data, 0, $file_size);
            $file_size = 0;
          }
          else{
            &ReceiveData(1024);
            $file_size -= 1024;
          }

          print FILEOUT $received_data;

          $packet_count++;

        }#while ($file_size > 0 && $aut_ok == 1)

        close (FILEOUT);
        `cp bild01.jpg bild02.jpg`;
        `mv bild01.tmp bild01.jpg`;
        &CopyIndicationPicture();

        $picture_count++;

        print "Pic no. " . $picture_count . "  " . $packet_count . " packets received\n";

        $send_data = "Received picture    ";

        &SendData($send_data, 20);  #$client_socket->send($send_data, 20);

        print "Sent: '" . $send_data . "'\n";
      }#if($file_size < 1){

    }#if($job_status !=0){

    &ReceiveData(20);
    print "received: >" . $received_data . "< \n";

    if($aut_ok == 1){

      if(substr($received_data, 0, 8) eq "Job done"){
        $battery_status = substr($received_data, 17, 3);
        print "battery_status=" . $battery_status . "\n";
        &WriteBatteryStatus($battery_status);
        
        if ($current_snapshot == 1){#snapshot function active?
          `cp bild01.jpg freezeimage.jpg`;
          $shell_command = time();
          $shell_command_02 = $shell_command;
          $shell_command =~ s/^ +//;
          $shell_command =~ s/ +$//;
          $shell_command .= ".jpg";
          $last_snapshot_name = $shell_command;
          $shell_command ="cp freezeimage.jpg snapshots/preview/" . $shell_command;
          `$shell_command`;
          `cp bild02.jpg freezeimage.jpg`;
          $shell_command = $shell_command_02;
          $shell_command =~ s/^ +//;
          $shell_command =~ s/ +$//;
          $shell_command .= ".jpg";
          $last_snapshot_name = $shell_command;
          $shell_command ="cp freezeimage.jpg snapshots/" . $shell_command;
          `$shell_command`;
          &CreateSnapshotFile();
          print "Snapshot copied\n";

        }
        $shell_command = "rm " . $file_command;
        `$shell_command`;
        $job_status=0;
      }

      if(substr($received_data, 0, 7) eq "Waiting"){#R2 has nothing to do
        print "Waiting for incomming command\n";
        $battery_status = substr($received_data, 17, 3);
        print "battery_status=" . $battery_status . "\n";
        &WriteBatteryStatus($battery_status);
        while($job_status == 0 && $aut_ok == 1){
          #Watch out for command
          &WatchDogFile(0);
          if (-e $file_command){#If command in pipline, start processing
            open(FILEIN, "<" . $file_command) || die "Can not open file " . $file_command . "!\n";
              $command_line = <FILEIN>;
            close(FILEIN);
            $current_user = substr($command_line, 60, 20);
            $current_ip = substr($command_line, 40, 20);
            $current_timestamp = substr($command_line, 20, 20);

            $start_command_time = substr($command_line, 20, 20);
            $last_action_time = time();
            $command_line = substr("Comm=" . $command_line, 0, 20);
            $drive_distance = substr($command_line, 7, 2);
            if($drive_distance < 0 | $drive_distance > 200){
              print "drive_distance=$drive_distance out of range!\n";
              $drive_distance = 0;
            }
            $total_drive_distance += $drive_distance * 1.4;
            if(substr($command_line, 14, 1) eq "1"){
              $current_snapshot = 1;
            }
            else{
              $current_snapshot = 0;
            }
            if($command_line ne "Comm="){
              &SendData($command_line, 20);  #$client_socket->send($command_line, 20);
              print "Sent: '" . $command_line . "'\n";
              $job_status = 1;
            }
            else{
              &LogFile("Empty command line in file '" . $file_command . "'");
            }
          }#if (-e $file_command){
          if($aut_ok && $job_status == 0 && (time() - $start_command_time > 300 || time() - $last_action_time > 30)){#Lease over after 5 minutes or if no command transmitted for 30 seconds
            if (-e $file_command){
              $shell_command = "rm " . $file_command;
              `$shell_command`;
            }
            #Clear file for R2 access to give new user the chance to take control
            open(FILEOUT, ">" . $file_access) || die "Can not open file " . $file_command . "!\n";
            close(FILEOUT);

            #Write information about current user to logfiles and create new statistics
            if($current_user ne ""){
              $last_snapshot_name = substr($last_snapshot_name . "                    ", 0, 20);
              @file_info = stat("access-history.log");
              if ($file_info[7] > 10000000){
                `mv access-history.log access-history01.log`;
              }
              open(FILEOUT, ">> access-history.log") || die "Can not open file access-history.log!\n";
                print FILEOUT $current_user . $current_ip . $current_timestamp . $last_snapshot_name . "_\n";
              close(FILEOUT);
              $total_user_count ++;
              open(FILEOUT, "> access-count.tmp") || die "Can not open file access-count.tmp!\n";
                print FILEOUT $total_user_count . "\n";
                print FILEOUT $total_drive_distance . "\n";
              close(FILEOUT);
              `cp access-count.tmp access-count.log`;
              &CreateHistoryFile();
            }
            $current_user = "";
            $current_ip = "";
            $current_timestamp = "";
            $current_snapshot = 0;
            $last_snapshot_name = "";
            $start_command_time = time();
            $wait_count++;
            print "wait_count = $wait_count\n";
            $send_data = "Still alive?        ";
            &SendData($send_data, 20);
            &ReceiveData(20);
            if(substr($received_data, 0, 4) ne "Yes!"){
              $aut_ok=0;
            }
            $battery_status = substr($received_data, 17, 3);
            print "battery_status=" . $battery_status . "\n";
            &WriteBatteryStatus($battery_status);
            $last_action_time = time();
          }
        }#while($job_status == 0){
      }
    }#if($aut_ok == 1){

  }# while($aut_ok == 1)
  close $client_socket;
  &LogFile("Client socked closed.");
}#while(111 == 111);

&FileLog("Server Script aborted?!?");


#++++++++++++++++++++ ReceiveData +++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub ReceiveData{
  if($aut_ok == 1){
    $received_data="";
    $client_socket->recv($received_data, $_[0], MSG_WAITALL);
    if(length($received_data) + 1 < $_[0]){
      &LogFile ("Receipt of data interrupted. received_data='" . $received_data . "' _[0]=" . $_[0] . ", length received_data=" . length($received_data));
      $received_data="";
      $aut_ok = 0;
      $job_status = 0;
      #die "Receipt of data interrupted.\n";
    }
  }
}

#++++++++++++++++++++ SendData +++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub SendData{
  if($aut_ok == 1){
    print "send_data='" . $_[0] . "'\n";
    eval {$client_socket->send($_[0], $_[1]);};
    if ($@) {
      &LogFile ("Sending of data interrupted. send_data='" . $send_data . "'");
      $aut_ok = 0;
      $job_status = 0;
    }
  }
}


#++++++++++++++++++++++ WriteBatteryStatus ++++++++++++++++++++++++++++++++++++++++++++++
sub WriteBatteryStatus{


  if($_[0] != $battery_status_old && $_[0] > 0){
    open(BATFILE, ">BatteryStatus.txt") || die "Can not open file 'BatteryStatus.txt'!\n";
      print BATFILE $_[0];
    close(BATFILE);
  }
  $battery_status_old = $_[0];

  @file_info=stat("R2-BatteryStatus.log");

  ($seconds, $minutes, $hours, $day_of_month, $month, $year, $weekday, $day_of_year, $summertime) = localtime(time());

  if ($hours < 10){
    $hours = "0" . $hours;
  }
  if ($minutes < 10){
    $minutes = "0" . $minutes;
  }
  if ($seconds < 10){
    $seconds = "0" . $seconds;
  }
  $month += 1;
  $year += 1900;

  if ($month < 10){
    $month = "0" . $month;
  }
  if ($day_of_month < 10){
    $day_of_month = "0" . $day_of_month;
  }

  $char_temp=substr($day_of_month . "." . $month . "." . $year . "-" . $hours . ":" . $minutes . ":" . $seconds . "                    ", 0, 20);

  if ($file_info[7] > 1000000){
    `mv R2-BatteryStatus.log R2-BatteryStatus01.log`
  }

  open(LOGFILE, ">> R2-BatteryStatus.log") || die "Can not open file 'R2-BatteryStatus.log'!\n";
    print LOGFILE $char_temp . $_[0] . "\n";
  close(LOGFILE);

}

#++++++++++++++++++++++ LogFile +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub LogFile{

  @file_info=stat("R2-server.log");

  ($seconds, $minutes, $hours, $day_of_month, $month, $year, $weekday, $day_of_year, $summertime) = localtime(time());

  if ($hours < 10){
    $hours = "0" . $hours;
  }
  if ($minutes < 10){
    $minutes = "0" . $minutes;
  }
  if ($seconds < 10){
    $seconds = "0" . $seconds;
  }
  $month += 1;
  $year += 1900;

  if ($month < 10){
    $month = "0" . $month;
  }
  if ($day_of_month < 10){
    $day_of_month = "0" . $day_of_month;
  }

  $char_temp=substr($day_of_month . "." . $month . "." . $year . "-" . $hours . ":" . $minutes . ":" . $seconds . "                    ", 0, 20);

  if ($file_info[7] > 10000){
    `mv R2-server.log R2-server01.log`
  }

  open(LOGFILE, ">> R2-server.log") || die "Can not open file 'R2-server.log'!\n";
    print LOGFILE $char_temp . $_[0] . "\n";
  close(LOGFILE);

}

#++++++++++++++++++++++ WatchDogFile +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub WatchDogFile{#Print a timestamp into a file to show watchdog-server.perl that script is still alive
  if(time() - $last_watchdog_call > 10){
    open(WATCHDOGFILE, "> watchdog.tmp") || die "Can not open file 'watchdog.tmp'!\n";
      print WATCHDOGFILE time() . "\n";
    close(WATCHDOGFILE);
    if($_[0] == 0){
      `mv watchdog.tmp watchdog.txt`;
    }
    $last_watchdog_call = time();
  }
}

#++++++++++++++++++++++ Copy indication for new picture +++++++++++++++++++++++++++++++++
sub CopyIndicationPicture{
  #There are 9 (very small) indication pictures (IndicationPicture01 - IndicationPicture09) stored at the server.
  #The width of those pictures alteres fom 1 to 9 pixel.
  #If a new picture from the R2 has been transmitted, CurentIndicationPicture is altered.
  #The Javascript of your browser compares the old width of CurentIndicationPicture with the new one, to indicate that a new R2 pic is at the server.
  #This keeps the transferrate of the picture download to your browser low.
  `cp IndicationPicture0$last_indication_pic_no.bmp CurrentIndicationPicture.bmp`;
  $last_indication_pic_no++;
  if($last_indication_pic_no == 10){
    $last_indication_pic_no = 1;
  }
}

#++++++++++++++++++++++ Create HTML file of the 40 last snapshots ++++++++++++++++++++++++
sub CreateSnapshotFile{
  my $files_count = 0;
  my $i = 0;
  my @file_list;
  my $max_number = 41;

  opendir(DIR,"snapshots");
  @file_list = readdir(DIR);
  closedir(DIR);

  @file_list = sort(@file_list);

  $files_count = scalar(@file_list);

  open(FILEOUT,'> R2-lastpics.tmp') || die "Can not open file lastpic.tmp!\n";

  print FILEOUT '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">';
  print FILEOUT  "\n";
  print FILEOUT '<html><head><meta http-equiv="pragma" content="no-cache" /><title>Last R2 Pics</title></head><body>';
  print FILEOUT  "\n";
  print FILEOUT '<h1>Click on preview image to see full resolution.</h1>';
  print FILEOUT  "\n";
  print FILEOUT '<h1>Klick auf das Vorschaubild, um die volle Aufl&ouml;sung aufzurufen.</h1>';
  print FILEOUT  "\n";

  if($files_count < $max_number){
    $max_number = $files_count + 1;
  }

  for($i=0; $i<$files_count; $i++){
    if($files_count - $i < $max_number){
      if(@file_list[2 * $files_count - $max_number - $i] != ""){
        print FILEOUT '<a href="snapshots/' . @file_list[2 * $files_count - $max_number - $i] . '"><img src="snapshots/preview/' . @file_list[2 * $files_count - $max_number - $i] . '"></a>' . "\n";
      }
    }
    else{
      `rm snapshots/@file_list[$i]`;
      `rm snapshots/preview/@file_list[$i]`;
    }
  }

  print FILEOUT "</body></html>\n";

  close (FILEOUT);

  `mv R2-lastpics.tmp R2-lastpics.htm`;


  print "...new picture gallery created.\n";
}

#++++++++++++++++++++++++++++++ Create lease history of the last 24 hours +++++++++++++++++++

sub CreateHistoryFile{
  for($i2=0; $i2 < 2; $i2++){
    $user_count = 0;

    if($i2 == 0){
      $file_target = "R2-access-history_en.htm";
    }
    if($i2 == 1){
      $file_target = "R2-access-history_ge.htm";
    }

    open(FILEIN, "< access-count.log") || die "Can not open file 'access-count.log' !\n";
      $total_user = <FILEIN>;
      $total_distance = <FILEIN>;
    close(FILEIN);

    $total_distance = sprintf ("%.0f", $total_distance);

    if (-e $file_in01){
      open(FILEIN, "<" . $file_in01) || die "Can not open file " . $file_in01 . "!\n";
      $i = 0;
    }
    else{
      if (-e $file_in02){
        open(FILEIN, "<" . $file_in02) || die "Can not open file " . $file_in02 . "!\n";
        $i = 1;
      }
      else{
        die "No input file found!\n";
      }
    }

    open(FILEOUT,'>'.$file_temp) || die "Can not open file " . $file_temp . "!\n";

    print FILEOUT '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">', "\n";
    print FILEOUT "<html><head><title>R2 user</title></head><body>\n";

    if($i2 == 0){
      print FILEOUT "Total number of leases since 1st June of 2011: <strong>" . $total_user . "</strong><br>\n";
      print FILEOUT "Covered distance: <strong>" . $total_distance . "cm</strong><br>\n";
      print FILEOUT "<br>\n";
      print FILEOUT "Drivers of the last 24 hours:<br>\n";
    }
    if($i2 == 1){
      print FILEOUT "Gesamtzahl R2-&quot;Vermietungen&quot; seit dem 1. Juni 2011: <strong>" . $total_user . "</strong><br>\n";
      print FILEOUT "Zur&uuml;ckgelegte Strecke: <strong>" . $total_distance . "cm</strong><br>\n";
      print FILEOUT "<br>\n";
      print FILEOUT "Fahrer der letzten 24 Stunden:<br>\n";
    }

    print FILEOUT '<table align="left"  border="1" cellpadding="5" cellspacing="0">' . "\n";

    if($i2 == 0){
      print FILEOUT '  <tr>' . "\n";
      print FILEOUT '    <td align="right">' . "\n";
      print FILEOUT "User no.\n";
      print FILEOUT '    </td>' . "\n";
      print FILEOUT '    <td>' . "\n";
      print FILEOUT "User\n";
      print FILEOUT '    </td>' . "\n";
      print FILEOUT '    <td>' . "\n";
      print FILEOUT "Time (CET)\n";
      print FILEOUT '    </td>' . "\n";
      print FILEOUT '    <td>' . "\n";
      print FILEOUT "Snapshot\n";
      print FILEOUT '    </td>' . "\n";
      print FILEOUT '  <tr>' . "\n";
    }
    if($i2 == 1){
      print FILEOUT '  <tr>' . "\n";
      print FILEOUT '    <td align="right">' . "\n";
      print FILEOUT "Benutzer Nr.\n";
      print FILEOUT '    </td>' . "\n";
      print FILEOUT '    <td>' . "\n";
      print FILEOUT "Name\n";
      print FILEOUT '    </td>' . "\n";
      print FILEOUT '    <td>' . "\n";
      print FILEOUT "Zeit (Mitteleuropa)\n";
      print FILEOUT '    </td>' . "\n";
      print FILEOUT '    <td>' . "\n";
      print FILEOUT "Foto\n";
      print FILEOUT '    </td>' . "\n";
      print FILEOUT '  <tr>' . "\n";
    }

    while($i<2){

      $line_in = <FILEIN>;

      while($line_in ne ""){

        $user = substr($line_in, 0, 20);
        $user =~ s/^ +//;
        $user =~ s/ +$//;

        $user_time = substr($line_in, 40, 20);
        $user_time =~ s/^ +//;
        $user_time =~ s/ +$//;

      if(time()-$user_time<3600 * 24){

        $user_count++;

        ($seconds, $minutes, $hours, $day_of_month, $month,
            $year, $weekday, $day_of_year, $summertime) = localtime($user_time);

          if ($hours < 10){
            $hours = "0" . $hours;
          }
          if ($minutes < 10){
            $minutes = "0" . $minutes;
          }
          if ($seconds < 10){
            $seconds = "0" . $seconds;
          }
          $month += 1;
          $year += 1900;

          if ($month < 10){
            $month = "0" . $month;
          }
          if ($day_of_month < 10){
            $day_of_month = "0" . $day_of_month;
          }

          $user_pic = substr($line_in, 60, 20);
          $user_pic =~ s/^ +//;
          $user_pic =~ s/ +$//;

          print FILEOUT '  <tr>' . "\n";
          print FILEOUT '    <td align="right">' . "\n";
          print FILEOUT $user_count . ".)\n";
          print FILEOUT '    </td>' . "\n";
          print FILEOUT '    <td>' . "\n";
          print FILEOUT $user . "\n";
          print FILEOUT '    </td>' . "\n";
          print FILEOUT '    <td>' . "\n";
          print FILEOUT $day_of_month . "." . $month . "." . $year . " - " . $hours . ":" . $minutes . ":" . $seconds . "\n";
          print FILEOUT '    </td>' . "\n";
          print FILEOUT '    <td>' . "\n";
          if($user_pic ne ""){
            print FILEOUT '<a href="http://www.RoboSpatium.de/R2/snapshot/' . $user_pic . '" target="_blank">' . $user_pic . "</a>\n";
          }
          else{
            print FILEOUT "&nbsp;\n";
          }
          print FILEOUT '    </td>' . "\n";
          print FILEOUT '  <tr>' . "\n";

        }#if(time()-$user_time<3600 * 12){

        $line_in = <FILEIN>;
      }

      close (FILEIN);

      if (-e $file_in02 && $i == 0){
        close (FILEIN);
        open(FILEIN, "<" . $file_in02) || die "Can not open file " . $file_in02 . "!\n";
        print "'$file_in02' opened\n";
        $i++;
      }
      else{
        if($i == 1){
          close (FILEIN);
        }
        $i = 1000;
      }

    }#while($i<2){

    print FILEOUT '</table>' . "\n";
    print FILEOUT "</body></html>\n";

    close (FILEOUT);

    `mv $file_temp $file_target`;
  }#for($i2=0; $i2 < 2; $i2++){


  print "...history file created.\n";
}
